Module: Minimization

Defined in:
lib/multidim/brent_root_finder.rb,
lib/minimization.rb,
lib/multidim/powell.rb,
lib/minimization/version.rb,
lib/multidim/nelder_mead.rb,
lib/multidim/point_value_pair.rb,
lib/multidim/conjugate_gradient.rb

Overview

brent_root_finder.rb -

Minimization- Minimization algorithms on pure Ruby Copyright © 2010 Claudio Bustos

This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.

This algorith was adopted and ported into Ruby from GNU GSL library’s brent.c [github.com/ampl/gsl/blob/master/roots/brent.c]. Therefore this file is under under the terms of the GNU General Public License. Research Paper - maths-people.anu.edu.au/~brent/pd/rpb005.pdf

Brent method for finding roots

Defined Under Namespace

Classes: Bisection, Brent, BrentRootFinder, ConjugateDirectionMinimizer, DirectSearchMinimizer, FletcherReeves, GoldenSection, NelderMead, NewtonRaphson, NonLinearConjugateGradientMinimizer, PointValuePair, PolakRibiere, Powell, Unidimensional

Constant Summary collapse

FailedIteration =
Class.new(Exception)
VERSION =
'0.2.5'